@@ -275,8 +275,11 @@ static int handle_name_req_complete(char *data, int sock)
         name[0] = '\0';
     }
     else {
+        char *end;
         ev->name[247] = '\0';
         strncpy(name, (char*)ev->name, sizeof(name));
+        if(!g_utf8_validate(name, -1, &end))
+            *end = '\0';
     }
 
     publish_dev(dev, name);
@@ -352,8 +355,10 @@ static int process_data(char *data, int data_len, int sock)
         switch (hdr->evt) {
             case EVT_INQUIRY_RESULT:
                 return handle_inquiry_result(body, sock);
+
             case EVT_INQUIRY_RESULT_WITH_RSSI:
                 return handle_inquiry_result_with_rssi(body, sock);
+
             case EVT_INQUIRY_COMPLETE:
                 inquiry_in_progress = FALSE;
 
@@ -377,9 +382,12 @@ static int process_data(char *data, int data_len, int sock)
                     else
                         return 1;
                 }
+
                 return 1;
+
             case EVT_REMOTE_NAME_REQ_COMPLETE:
                 return handle_name_req_complete(body, sock);
+
             case EVT_CMD_STATUS:
                 {
                     evt_cmd_status *ev = (evt_cmd_status *)body;
@@ -395,6 +403,21 @@ static int process_data(char *data, int data_len, int sock)
                     }
                     return 0;
                 }
+
+            case EVT_STACK_INTERNAL:
+                {
+                    evt_stack_internal *si = (evt_stack_internal *) body;
+                    evt_si_device *sd = (evt_si_device *) &si->data;
+
+                    if (sd->event == HCI_DEV_DOWN) {
+                        inquiry_in_progress = FALSE;
+                        name_req_in_progress = FALSE;
+                        return 1;
+                    }
+                }
+                
+                return 0;
+
             default:
                 debug("Invalid event type (0x%02x)", hdr->evt);
                 return 1;
@@ -460,6 +483,7 @@ gboolean inquiry_start(void)
                           EVT_INQUIRY_COMPLETE,
                           EVT_CMD_STATUS,
                           EVT_REMOTE_NAME_REQ_COMPLETE,
+                          EVT_STACK_INTERNAL,
                           0 };
 
     hci_sock = event_socket(hci_events);
